<?php

/*

	Plugin Name: Text Obfuscator
	Plugin URI: http://www.flutt.co.uk/development/wordpress-plugins/text-obfuscator/
	Version: 1.2
	Description: A simple plugin for replacing words or phrases in your posts' content with different words or phrases.  Initially designed for protecting the innocent by removing names from personal blog posts, it can also be used to correct common spelling errors or expand abbreviations.  The plugin operates on output preserving the content in the database as entered in the post.
	Author: ConfuzzledDuck
	Author URI: http://www.flutt.co.uk/

*/

#
#  text-obfuscator.php
#
#  Created by Jonathon Wardman on 04-10-2008.
#  Copyright 2008 - 2011, Jonathon Wardman. All rights reserved.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  You may obtain a copy of the License at:
#  http://www.gnu.org/licenses/gpl-3.0.txt
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

 /**
  * Main Syndicate Out Plugin functionality.
  */

	 // Add actions for admin pages...
if ( is_admin() ) {
	add_action( 'admin_menu', 'obfuscator_add_pages' );
	add_action( 'admin_init', 'obfuscator_register_settings' );
}

	 // Add actions for main post output parsing functionality...
add_filter( 'the_content', 'obfuscator_filter', 10 );
add_filter( 'the_title', 'obfuscator_filter_title', 10 );
add_filter( 'single_post_title', 'obfuscator_filter_title', 10 );
add_filter( 'the_excerpt', 'obfuscator_filter_excerpt', 10 );

	 // Add actions for main post input parsing functionality...
add_filter( 'content_save_pre', 'obfuscator_filter_input_content', 10 );
add_filter( 'title_save_pre', 'obfuscator_filter_input_title', 10 );
add_filter( 'excerpt_save_pre', 'obfuscator_filter_input_excerpt', 10 );

	 // Add actions for comment parsing functionality...
add_filter( 'comment_text', 'obfuscator_filter_comment', 10 );
add_filter( 'pre_comment_content', 'obfuscator_filter_input_comment', 10 );
add_filter( 'comment_excerpt', 'obfuscator_filter_comment', 10 );

 /**
  * Adds the main admin management page.
  *
  * @since 0.2
  */
function obfuscator_add_pages() {

	add_management_page( 'Text Obfuscator admin', 'Text Obfuscator', 2, 'obfuscator', 'obfuscator_admin_page' );

}

 /**
  * Registers the required settings array.
  *
  * @since 0.2
  */
function obfuscator_register_settings() {

	register_setting( 'text-obfuscator', 'obfuscator_replacements', 'obfuscator_sanitize_options' );

}

 /**
  * Sanitizes replacement pairs.
  *
  * @since 0.2
  */
function obfuscator_sanitize_options( $data ) {

	$sanitizedData = array();
	foreach ( $data AS $replacementPair ) {
		if ( isset( $replacementPair['token'] ) && !empty( $replacementPair['token'] ) && isset( $replacementPair['value'] ) && !empty( $replacementPair['value'] ) ) {
			$sanitizedData[] = array( 'token' => $replacementPair['token'],
			                          'value' => $replacementPair['value'],
			                          'location' => ( isset( $replacementPair['location'] ) && ( 'pre' == $replacementPair['location'] ) ) ? 'pre' : 'post',
			                          'scope' => ( isset( $replacementPair['scope'] ) && ( 'part' == $replacementPair['scope'] ) ) ? 'part' : 'full',
			                          'posts' => ( isset( $replacementPair['posts'] ) ) ? true : false,
			                          'titles' => ( isset( $replacementPair['titles'] ) ) ? true : false,
			                          'excerpts' => ( isset( $replacementPair['excerpts'] ) ) ? true : false,
			                          'comments' => ( isset( $replacementPair['comments'] ) ) ? true : false );
		}
	}
	
	return $sanitizedData;

}

 /**
  * Outputs the admin page.
  *
  * @since 0.2
  */
function obfuscator_admin_page() {

	$pairModifier = 1;
	if ( isset( $_POST['add_pairs'] ) && is_numeric( $_POST['add_pairs'] ) ) {
		$pairModifier = $_POST['new_pairs'];
	}
	$replacementData = get_option( 'obfuscator_replacements' );

?>
<div class="wrap">
<form id="posts-filter" action="options.php" method="post">
	<?php settings_fields( 'text-obfuscator' ); ?>
	<h2>Text Obfuscator admin</h2>
	<?php if ( isset( $_GET['updated'] ) ) { echo '<div id="message" class="updated fade"><p>Your replacement rules have been updated and applied to all published posts.</p></div>'; } ?>
	<p>Add the terms you want replacing into the boxes in the 'Original text'
	column and the text you want that term replacing with into the boxes in the
	'Replacement' column. Select when you want the replacement to take place and
	check the boxes to select which bits of content you want the filter to apply
	to. To remove a rule simply delete the content from both boxes for that
	line; if it's a display filter the original text will appear, if it's a save
	filter the replaced text will remain.</p>
	<table class="form-table">
		<tr valign="top">
			<th scope="column">Original Text<br /><span class="description">Text to to be replaced</span></th>
			<th scope="column">Replacement<br /><span class="description">Text to put in it's place</span></th>
			<th scope="column">Point<br /><span class="description">On save or display</span></th>
			<th scope="column">Match<br /><span class="description">Full or part words</span></th>
			<th scope="column">Posts<br /><span class="description">Replace in body</span></th>
			<th scope="column">Titles<br /><span class="description">Replace in title</span></th>
			<th scope="column">Excerpt<br /><span class="description">Replace in excerpt</span></th>
			<th scope="column">Comments<br /><span class="description">Replace in comments</span></th>
		</tr>
<?php
	$tokenCount = count( $replacementData );
	for ( $i = 0; $i < ( $tokenCount + $pairModifier ); $i++ ) {
?>
		<tr valign="top">
			<td><input type="text" name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][token]" value="<?php echo esc_attr( $replacementData[$i]['token'] ); ?>" /></td>
			<td><input type="text" name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][value]" value="<?php echo esc_attr( $replacementData[$i]['value'] ); ?>" /></td>
			<td>
				<select name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][location]">
					<option value="post" <?php echo ( isset( $replacementData[$i]['location'] ) && 'post' == $replacementData[$i]['location'] ) ? 'selected="selected"' : ''; ?>>Display</option>
					<option value="pre" <?php echo ( isset( $replacementData[$i]['location'] ) && 'pre' == $replacementData[$i]['location'] ) ? 'selected="selected"' : ''; ?>>Save</option>
				</select>
			</td>
			<td>
				<select name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][scope]">
					<option value="full" <?php echo ( isset( $replacementData[$i]['scope'] ) && 'full' == $replacementData[$i]['scope'] ) ? 'selected="selected"' : ''; ?>>Full</option>
					<option value="part" <?php echo ( isset( $replacementData[$i]['scope'] ) && 'part' == $replacementData[$i]['scope'] ) ? 'selected="selected"' : ''; ?>>Part</option>
				</select>
			</td>
			<td><input type="checkbox" name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][posts]" value="true" <?php echo ( isset( $replacementData[$i]['posts'] ) && true == $replacementData[$i]['posts'] ) ? 'checked="checked"' : ''; ?> /></td>
			<td><input type="checkbox" name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][titles]" value="true" <?php echo ( isset( $replacementData[$i]['titles'] ) && true == $replacementData[$i]['titles'] ) ? 'checked="checked"' : ''; ?> /></td>
			<td><input type="checkbox" name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][excerpts]" value="true" <?php echo ( isset( $replacementData[$i]['excerpts'] ) && true == $replacementData[$i]['excerpts'] ) ? 'checked="checked"' : ''; ?> /></td>
			<td><input type="checkbox" name="obfuscator_replacements[<?php echo esc_attr( $i ); ?>][comments]" value="true" <?php echo ( isset( $replacementData[$i]['comments'] ) && true == $replacementData[$i]['comments'] ) ? 'checked="checked"' : ''; ?> /></td>
		</tr>
<?php	
	}
?>
	</table>
	<!--<p>Add <input type="text" size="3" value="1" name="new_pairs" /> new pair(s) <input type="submit" name="add_pairs" value="Go" class="button" /></p>-->
	<p class="submit">
		<input type="submit" name="save_pairs" value="Save changes / add blank row" class="button-primary" />
	</p>
</form>
</div>
<?php

}

 /**
  * Builds the regular expressions needed for replacement for the given token
  * value pair.
  *
  * @since 1.2
  */
function obfuscator_build_replacement_elements( $replacementItem ) {

	if ( isset( $replacementItem['token'] ) && isset( $replacementItem['value'] ) ) {
		if ( 'part' == $replacementItem['scope'] ) {
			return array( 'token' => '/'.preg_quote( $replacementItem['token'], '/' ).'/i',
			              'value' => $replacementItem['value'] );
		} else {
			return array( 'token' => '/([\W\s]?)'.preg_quote( $replacementItem['token'], '/' ).'([\W\s]+)/i',
			              'value' => '\\1'.$replacementItem['value'].'\\2' );
		}
	} else {
		return false;
	}

}

 /**
  * Carries out the string replacement on post output.
  *
  * @since 0.2
  */
function obfuscator_filter( $content, $content_type = 'posts' ) {

	$replacementData = get_option( 'obfuscator_replacements' );
	$replacementTokens = $replacementValues = array();
	foreach ( $replacementData AS $replacementItem ) {
		if ( ! isset( $replacementItem['location'] ) || ( 'post' == $replacementItem['location'] ) ) {
			if ( ! isset( $replacementItem[$content_type] ) || ( true == $replacementItem[$content_type] ) ) {
				if ( $replacementElements = obfuscator_build_replacement_elements( $replacementItem ) ) {
					$replacementTokens[] = $replacementElements['token'];
					$replacementValues[] = $replacementElements['value'];
				}
			}
		}
	}
	return preg_replace( $replacementTokens, $replacementValues, $content );

}

	 // Brief output function aliases...
function obfuscator_filter_content( $content ) { return obfuscator_filter( $content, 'posts' ); }
function obfuscator_filter_title( $content ) { return obfuscator_filter( $content, 'titles' ); }
function obfuscator_filter_excerpt( $content ) { return obfuscator_filter( $content, 'excerpts' ); }
function obfuscator_filter_comment( $content ) { return obfuscator_filter( $content, 'comments' ); }

 /**
  * Carries out the string replacement on post input.
  *
  * @since 1.1
  */
function obfuscator_filter_input( $content, $content_type = 'posts' ) {

	$replacementData = get_option( 'obfuscator_replacements' );
	$replacementTokens = $replacementValues = array();
	foreach ( $replacementData AS $replacementItem ) {
		if ( isset( $replacementItem['location'] ) && ( 'pre' == $replacementItem['location'] ) ) {
			if ( ! isset( $replacementItem[$content_type] ) || ( true == $replacementItem[$content_type] ) ) {
				if ( $replacementElements = obfuscator_build_replacement_elements( $replacementItem ) ) {
					$replacementTokens[] = $replacementElements['token'];
					$replacementValues[] = $replacementElements['value'];
				}
			}
		}
	}
	return preg_replace( $replacementTokens, $replacementValues, $content );

}

	 // Brief input function aliases...
function obfuscator_filter_input_content( $content ) { return obfuscator_filter_input( $content, 'posts' ); }
function obfuscator_filter_input_title( $content ) { return obfuscator_filter_input( $content, 'titles' ); }
function obfuscator_filter_input_excerpt( $content ) { return obfuscator_filter_input( $content, 'excerpts' ); }
function obfuscator_filter_input_comment( $content ) { return obfuscator_filter_input( $content, 'comments' ); }